##
# File:  MileStoneFile.py
# Date:  18-May-2015
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import os
import sys

# from mmcif_utils.trans.InstanceMapper       import InstanceMapper
# from wwpdb.utils.config.ConfigInfo import ConfigInfo
# from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppEm
from wwpdb.io.file.DataExchange import DataExchange
from wwpdb.io.locator.PathInfo import PathInfo


class MileStoneFile(object):
    """
    """
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
        """
        self.__reqObj = reqObj
        self.__lfh = log
        self.__verbose = verbose
        self.__sObj = self.__reqObj.newSessionObj()
        # self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        self.__siteId = str(self.__reqObj.getValue("WWPDB_SITE_ID"))
        self.__inputFile = ''
        # self.__isEmEntry = False
        # self.__sessionFile = ''
        self.__pI = PathInfo(siteId=self.__siteId, sessionPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)

    def setEmEntry(self):
        """
        """
        # self.__isEmEntry = True
        pass  # We do not care anymore - but leave API  pylint: disable=unnecessary-pass

    def getInputFile(self, entryId, source, content, format, version):  # pylint: disable=redefined-builtin
        """
        """
        sourcePath = self.__pI.getFilePath(dataSetId=entryId, wfInstanceId=None, contentType=content, formatType=format,
                                           fileSource=source, versionId=version, partNumber='1')
        if os.access(sourcePath, os.F_OK):
            self.__inputFile = sourcePath
            return True
        #
        return False

    def getOutputFile(self, entryId, source, content, format, version):  # pylint: disable=redefined-builtin
        """
        """
        if not self.__inputFile:
            return False
        #
        # if self.__isEmEntry:
        #     self.__cIA=ConfigInfoAppEm(self.__siteId)
        #     im = InstanceMapper(verbose=self.__verbose, log=self.__lfh)
        #     im.setMappingFilePath(self.__cIA.get_emd_mapping_file_path())
        #     self.__sessionFile = os.path.join(self.__sessionPath, entryId + '.emd.cif')
        #     im.translate(self.__inputFile, self.__sessionFile, mode="src-dst")
        #
        dE = DataExchange(reqObj=self.__reqObj, depDataSetId=entryId, fileSource=source, verbose=self.__verbose, log=self.__lfh)
        # if self.__sessionFile and os.access(self.__sessionFile, os.F_OK):
        #     dE.export(self.__sessionFile, contentType=content, formatType=format, version=version)
        # else:
        dE.export(self.__inputFile, contentType=content, formatType=format, version=version)
        #
        sourcePath = self.__pI.getFilePath(dataSetId=entryId, wfInstanceId=None, contentType=content, formatType=format,
                                           fileSource=source, versionId='latest', partNumber='1')
        if os.access(sourcePath, os.F_OK):
            return True
        #
        return False



##
# File:  BaseClass.py
# Date:  24-Apr-2017
# Updates:
##
"""
Base class for handle all workflow manager task activities

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2017 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V1.00"

try:
    import cPickle as pickle
except ImportError:
    import pickle as pickle

import filecmp
import os
import shutil
import sys

from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppCommon
from wwpdb.io.locator.PathInfo import PathInfo


class BaseClass(object):
    """ Base Class responsible for all workflow manager task activities
    """
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        self._verbose = verbose
        self._lfh = log
        self._reqObj = reqObj
        self._sObj = None
        self._sessionId = None
        self._sessionPath = None
        self._siteId = str(self._reqObj.getValue("WWPDB_SITE_ID"))
        self._cI = ConfigInfo(self._siteId)
        self._cICommon = ConfigInfoAppCommon(self._siteId)
        self._archivePath = self._cI.get('SITE_ARCHIVE_STORAGE_PATH')
        #
        self.__getSession()
        self._pI = PathInfo(siteId=self._siteId, sessionPath=self._sessionPath, verbose=self._verbose, log=self._lfh)

    def _getExistingArchiveFile(self, entryId, contentType, formatType, version):
        filePath = self._findArchiveFileName(entryId, contentType, formatType, version)
        message = ''
        if not filePath:
            message = "Can not find " + version + " " + contentType + " " + formatType + " file."
        elif not os.access(filePath, os.F_OK):
            message = "File " + filePath + " does not exist."
        #
        return message, filePath

    def _findArchiveFileName(self, entryId, contentType, formatType, version):
        return self._pI.getFilePath(dataSetId=entryId, wfInstanceId=None, contentType=contentType, formatType=formatType,
                                    fileSource='archive', versionId=version, partNumber='1')

    def _copyFileUtil(self, sourcePath, targetPath):
        if not sourcePath:
            return 'Copying file: No source file defined.'
        #
        if not targetPath:
            return 'Copying file: No target file defined.'
        #
        if not os.access(sourcePath, os.F_OK):
            return "File '" + sourcePath + "' not found."
        #
        shutil.copyfile(sourcePath, targetPath)
        if not os.access(targetPath, os.F_OK):
            return "copying '" + sourcePath + "' to '" + targetPath + "' failed."
        elif not filecmp.cmp(sourcePath, targetPath):
            return "copying '" + sourcePath + "' to '" + targetPath + "' failed."
        #
        return ''

    def _bashSetting(self):
        setting = " RCSBROOT=" + self._cICommon.get_site_annot_tools_path() + "; export RCSBROOT; " \
            + " COMP_PATH=" + self._cICommon.get_site_cc_cvs_path() + "; export COMP_PATH; " \
            + " BINPATH=${RCSBROOT}/bin; export BINPATH; " \
            + " LOCALBINPATH=" + os.path.join(self._cICommon.get_site_local_apps_path(), 'bin') + "; export LOCALBINPATH; " \
            + " DICTBINPATH=" + os.path.join(self._cICommon.get_site_packages_path(), 'dict', 'bin') + "; export DICTBINPATH; "
        return setting

    def _getCmd(self, command, inputFile, outputFile, logFile, clogFile, extraOptions):
        cmd = "cd " + self._sessionPath + " ; " + self._bashSetting() + " " + command
        if inputFile:
            cmd += " -input " + inputFile
        #
        if outputFile:
            if outputFile != inputFile:
                self._removeFile(os.path.join(self._sessionPath, outputFile))
            #
            cmd += " -output " + outputFile
        #
        if extraOptions:
            cmd += " " + extraOptions
        #
        if logFile:
            self._removeFile(os.path.join(self._sessionPath, logFile))
            cmd += " -log " + logFile
        #
        if clogFile:
            self._removeFile(os.path.join(self._sessionPath, clogFile))
            cmd += " > " + clogFile + " 2>&1"
        #
        cmd += " ; "
        return cmd

    def _runCmd(self, cmd):
        # self._lfh.write('running cmd=%s\n' % cmd)
        os.system(cmd)

    def _removeFile(self, filePath):
        if os.access(filePath, os.F_OK):
            os.remove(filePath)
        #

    def _dumpPickle(self, pickleFile, pickleData):
        fb = open(os.path.join(self._sessionPath, pickleFile + '.pickle'), 'wb')
        pickle.dump(pickleData, fb)
        fb.close()

    def _loadPickle(self, pickleFile):
        picklePath = os.path.join(self._sessionPath, pickleFile + '.pickle')
        if not os.access(picklePath, os.F_OK):
            return None
        #
        fb = open(picklePath, 'rb')
        pickleData = pickle.load(fb)
        fb.close()
        return pickleData

    def _removePickle(self, pickleFile):
        self._removeFile(os.path.join(self._sessionPath, pickleFile + '.pickle'))

    def _processTemplate(self, fn, parameterDict=None):
        """ Read the input HTML template data file and perform the key/value substitutions in the
            input parameter dictionary.

            :Params:
                ``parameterDict``: dictionary where
                key = name of subsitution placeholder in the template and
                value = data to be used to substitute information for the placeholder

            :Returns:
                string representing entirety of content with subsitution placeholders now replaced with data
        """
        if parameterDict is None:
            parameterDict = {}
        tPath = self._reqObj.getValue("TemplatePath")
        fPath = os.path.join(tPath, fn)
        ifh = open(fPath, 'r')
        sIn = ifh.read()
        ifh.close()
        return (sIn % parameterDict)

    def _getLogMessage(self, logfile):
        if not os.access(logfile, os.F_OK):
            return ''
        #
        f = open(logfile, 'r')
        data = f.read()
        f.close()
        #
        msg = ''
        t_list = data.split('\n')
        for line in t_list:
            if not line:
                continue
            #
            if line == 'Finished!':
                continue
            #
            if msg:
                msg += '\n'
            #
            msg += line
        #
        return msg

    def __getSession(self):
        """ Join existing session or create new session as required.
        """
        #
        self._sObj = self._reqObj.newSessionObj()
        self._sessionId = self._sObj.getId()
        self._sessionPath = self._sObj.getPath()
        if (self._verbose):
            self._lfh.write("------------------------------------------------------\n")
            self._lfh.write("+BaseClass._getSession() - creating/joining session %s\n" % self._sessionId)
            self._lfh.write("+BaseClass._getSession() - session path %s\n" % self._sessionPath)
        #



##
# File:  CopyFileToAutoGroup.py
# Date:  29-Jun-2016
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import multiprocessing
import os
import shutil
import sys

from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.utils.config.ConfigInfoApp import ConfigInfoAppCommon
from wwpdb.io.locator.DataReference import DataFileReference
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
from wwpdb.io.locator.PathInfo import PathInfo
#


class CopyFileToAutoGroup(object):
    def __init__(self, reqObj=None, entryList=None, verbose=False, log=sys.stderr):
        """
        """
        self.__reqObj = reqObj
        self.__entryList = entryList
        self.__verbose = verbose
        self.__lfh = log
        self.__siteId = str(self.__reqObj.getValue('WWPDB_SITE_ID'))
        self.__cI = ConfigInfo(self.__siteId)
        self.__cICommon = ConfigInfoAppCommon(self.__siteId)
        self.__archivePath = self.__cI.get('SITE_ARCHIVE_STORAGE_PATH')
        self.__identifier = str(self.__reqObj.getValue('identifier'))
        self.__targetPath = os.path.join(self.__archivePath, 'autogroup', self.__identifier, 'processed')
        #
        self.__rcsbRoot = self.__cICommon.get_site_annot_tools_path()
        self.__compRoot = self.__cICommon.get_site_cc_cvs_path()
        self.__dictBinRoot = os.path.join(self.__cICommon.get_site_packages_path(), 'dict', 'bin')
        self.__dictRoot = self.__cICommon.get_mmcif_dict_path()
        self.__dictionary_v40 = self.__cI.get('SITE_PDBX_V4_DICT_NAME') + '.sdb'
        self.__dictionary_v5 = self.__cICommon.get_mmcif_archive_next_dict_filename() + '.sdb'
        #
        # self.__sessionId = None
        self.__sessionPath = None
        #
        self.__dfRef = DataFileReference(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        self.__dfRef.setStorageType('session')
        self.__dfRef.setPartitionNumber("1")
        self.__dfRef.setVersionId("none")
        #
        self.__getSession()
        self.__pI = PathInfo(siteId=self.__siteId, sessionPath=self.__sessionPath, verbose=self.__verbose, log=self.__lfh)
        #
        self.__fileTypeList = [ [ 'model',                  'pdbx' ],  # noqa: E201,E202,E241
                                [ 'structure-factors',      'pdbx' ],  # noqa: E201,E202,E241
                                [ 'validation-report-full', 'pdf'  ],  # noqa: E201,E202,E241
                                [ 'validation-report',      'pdf'  ],  # noqa: E201,E202,E241
                                [ 'validation-data',        'xml'  ] ]  # noqa: E201,E202,E241
        #

    def run(self):
        """
        """
        numProc = int(multiprocessing.cpu_count() / 2)
        mpu = MultiProcUtil(verbose=True)
        mpu.set(workerObj=self, workerMethod="runMulti")
        mpu.setWorkingDir(self.__sessionPath)
        _ok, _failList, _retLists, _diagList = mpu.runMulti(dataList=self.__entryList, numProc=numProc, numResults=1)
        return self.__getReturnMessage()

    def runMulti(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
        """
        """
        rList = []
        for entry_id in dataList:
            processedPath = os.path.join(self.__targetPath, entry_id)
            if not os.path.isdir(processedPath):
                os.mkdir(processedPath)
            #
            self.__dfRef.setSessionPath(processedPath)
            self.__dfRef.setSessionDataSetId(entry_id)
            for fileType in self.__fileTypeList:
                sourceFile = self.__pI.getFilePath(dataSetId=entry_id, wfInstanceId=None, contentType=fileType[0], formatType=fileType[1],
                                                   fileSource='archive', versionId='latest', partNumber='1')
                if (not sourceFile) or (not os.access(sourceFile, os.F_OK)):
                    continue
                #
                self.__dfRef.setContentTypeAndFormat(fileType[0], fileType[1])
                targetFile = self.__dfRef.getFilePathReference()
                if fileType[0] == 'model' and fileType[1] == 'pdbx':
                    self.__generateV4PdbxFile(sourceFile, entry_id + '.cif', entry_id + '.cif_log')
                    cifFile = os.path.join(self.__sessionPath, entry_id + '.cif')
                    if os.access(cifFile, os.F_OK):
                        shutil.copyfile(cifFile, targetFile)
                    #
                    self.__generatePdbFile(sourceFile, entry_id + '.pdb', entry_id + '.log', entry_id + '.pdb_log')
                    pdbFile = os.path.join(self.__sessionPath, entry_id + '.pdb')
                    if os.access(pdbFile, os.F_OK):
                        self.__dfRef.setContentTypeAndFormat(fileType[0], 'pdb')
                        targetPdbFile = self.__dfRef.getFilePathReference()
                        shutil.copyfile(pdbFile, targetPdbFile)
                    #
                else:
                    shutil.copyfile(sourceFile, targetFile)
                #
            #
            rList.append(entry_id)
        #
        return rList, rList, []

    def __getSession(self):
        """
        """
        self.__sObj = self.__reqObj.newSessionObj()
        # self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()

    def __generateV4PdbxFile(self, inputFile, outputFileName, commandLogFile):
        """
        """
        cmd = 'cd ' + self.__sessionPath + ' ; ' + self.__dictBinRoot + '/cifexch2 -dicSdb ' + os.path.join(self.__dictRoot, self.__dictionary_v5) \
            + ' -pdbxDicSdb ' + os.path.join(self.__dictRoot, self.__dictionary_v40) + ' -reorder -strip -op in -pdbids -input ' \
            + inputFile + ' -output ' + outputFileName + ' > ' + commandLogFile + ' 2>&1 ; '
        os.system(cmd)

    def __generatePdbFile(self, inputFile, outputFileName, outputLogFile, commandLogFile):
        """
        """
        cmd = 'cd ' + self.__sessionPath + ' ; RCSBROOT=' + self.__rcsbRoot + ' ; export RCSBROOT ; COMP_PATH=' + self.__compRoot \
            + ' ; export COMP_PATH ; BINPATH=${RCSBROOT}/bin; export BINPATH; ${BINPATH}/maxit -i ' + inputFile + ' -o 2 -output ' \
            + outputFileName + ' -log ' + outputLogFile + ' > ' + commandLogFile + ' 2>&1 ; '
        os.system(cmd)

    def __getReturnMessage(self):
        """
        """
        message = ''
        for entry_id in self.__entryList:
            processedPath = os.path.join(self.__targetPath, entry_id)
            self.__dfRef.setSessionPath(processedPath)
            self.__dfRef.setSessionDataSetId(entry_id)
            found = False
            for fileType in self.__fileTypeList:
                sourceFile = self.__pI.getFilePath(dataSetId=entry_id, wfInstanceId=None, contentType=fileType[0], formatType=fileType[1],
                                                   fileSource='archive', versionId='latest', partNumber='1')
                if (not sourceFile) or (not os.access(sourceFile, os.F_OK)):
                    continue
                #
                self.__dfRef.setContentTypeAndFormat(fileType[0], fileType[1])
                targetFile = self.__dfRef.getFilePathReference()
                if os.access(targetFile, os.F_OK):
                    found = True
                else:
                    message += 'Missing ' + targetFile + '\n'
                #
                if fileType[0] == 'model' and fileType[1] == 'pdbx':
                    self.__dfRef.setContentTypeAndFormat(fileType[0], 'pdb')
                    targetPdbFile = self.__dfRef.getFilePathReference()
                    if os.access(targetPdbFile, os.F_OK):
                        found = True
                    else:
                        message += 'Missing ' + targetPdbFile + '\n'
                    #
                #
            #
            if found:
                message += 'Copied entry ' + entry_id + '\n'
            #
        #
        return message


def main_copy():
    from wwpdb.utils.session.WebRequest import InputRequest
    siteId = 'WWPDB_DEPLOY_TEST_RU'
    os.environ["WWPDB_SITE_ID"] = siteId
    cI = ConfigInfo(siteId)
    #
    myReqObj = InputRequest({}, verbose=True, log=sys.stderr)
    myReqObj.setValue("TopSessionPath", cI.get('SITE_WEB_APPS_TOP_SESSIONS_PATH'))
    myReqObj.setValue("WWPDB_SITE_ID", siteId)
    myReqObj.setValue("identifier", "G_1002003")
    myReqObj.setValue("sessionid", " b8030220bdf3559c10a2c63618cd85a25256f7c1")
    entryList = ['D_8000200001', 'D_8000200003', 'D_8000200027', 'D_8000200040']
    copyUtil = CopyFileToAutoGroup(reqObj=myReqObj, entryList=entryList, verbose=False, log=sys.stderr)
    copyUtil.run()


if __name__ == '__main__':
    main_copy()



##
# File:  DepictOther.py
# Date:  25-Mar-2016
#
# Updates:
#  09-Dec-2024  zf   call _getPdbExtIdMap() method to get 'ext_pdb_id'
#
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import sys
try:
    from urllib.parse import quote as u_quote
except ImportError:
    from urllib import quote as u_quote

from wwpdb.utils.wf.dbapi.WFEtime import getTimeFromEpoc
from wwpdb.utils.session.FileUtils import FileUtils
from wwpdb.apps.workmanager.depict.DepictBase import DepictBase, processPublicIDs
from wwpdb.apps.workmanager.depict.DepictWorkFlow import DepictWorkFlow
from wwpdb.apps.workmanager.file_access.LogFileUtil import LogFileUtil


class DepictOther(DepictBase):
    """
    """
    def __init__(self, reqObj=None, statusDB=None, conFigObj=None, verbose=False, log=sys.stderr):
        """
        """
        super(DepictOther, self).__init__(reqObj=reqObj, statusDB=statusDB, conFigObj=conFigObj, verbose=verbose, log=log)
        #
        self.__dataD = {}
        self.__lastInst = {}
        self.__setup()

    def SummaryPage(self):
        """
        """
        self._dataInfo['summary_tmplt'] = [self.__dataD]
        #
        self._connectContentDB()
        c_authors = self._contentDB.ContactAuthor(depositionid=self._reqObj.getValue("identifier"))
        if c_authors:
            self._dataInfo['contact_table_row_tmplt'] = list(c_authors)
        #
        audit_history = self._contentDB.AuditHistory(depositionid=self._reqObj.getValue("identifier"))
        if audit_history:
            hisList = []
            for Dict in audit_history:
                myD = {}
                for item in ('public_version', 'date', 'file_version', 'description'):
                    myD[item] = ''
                #
                if ('major_revision' in Dict) and ('minor_revision' in Dict):
                    myD['public_version'] = str(Dict['major_revision']) + '.' + str(Dict['minor_revision'])
                #
                if ('revision_date' in Dict) and Dict['revision_date']:
                    myD['date'] = Dict['revision_date'].strftime("%b. %d, %Y")
                #
                if ('internal_version' in Dict) and Dict['internal_version']:
                    myD['file_version'] = 'V' + str(Dict['internal_version'])
                #
                if ('description' in Dict) and Dict['description']:
                    myD['description'] = Dict['description']
                #
                hisList.append(myD)
            #
            self._dataInfo['audit_history_table_row_tmplt'] = hisList
        #
        timestamps = self._statusDB.getTimeStampInfo(depositionid=self._reqObj.getValue("identifier"))
        if timestamps:
            self._dataInfo['timestamp_table_row_tmplt'] = list(timestamps)
            for tdir in self._dataInfo['timestamp_table_row_tmplt']:
                if ('mtime' in tdir) and tdir['mtime']:
                    tdir['mtime'] = getTimeFromEpoc(tdir['mtime'])
                #
            #
        #

    def ReplacementPage(self):
        """
        """
        self._connectContentDB()

        replace_counts = self._contentDB.GetReplaceCounts()
        if replace_counts:
            replaceList = []
            for Dict in replace_counts:
                myD = {}
                for item in ('ORCID', 'numreplace', 'name'):
                    myD[item] = ''
                #
                if 'identifier_ORCID' in Dict:
                    myD['ORCID'] = str(Dict['identifier_ORCID'])
                #
                if 'numreplace' in Dict:
                    myD['numreplace'] = str(Dict['numreplace'])
                #
                if 'name' in Dict:
                    myD['name'] = str(Dict['name'])
                #
                replaceList.append(myD)
                #
            self._dataInfo['replace_count_table_row_tmplt'] = replaceList

    def AllFilePage(self):
        """
        """
        fu = FileUtils(self._reqObj.getValue("identifier"), reqObj=self._reqObj, verbose=self._verbose, log=self._lfh)
        for item in ('archive', 'deposit', 'wf-instance'):
            _nFiles, list = fu.renderFileList(fileSource=item)  # pylint: disable=redefined-builtin
            if item == 'wf-instance':
                self.__dataD['wf_instance'] = '\n'.join(list)
            else:
                self.__dataD[item] = '\n'.join(list)
            #
        #
        self._dataInfo['allfile_tmplt'] = [self.__dataD]

    def getLevelPageSetting(self, page_tmplt):
        """
        """
        self.__getLastInst()
        method = self.__getInstanceData('dep_exp_method')
        if method:
            self._reqObj.setValue('method', method)
            self._reqObj.setValue("urlmethod", u_quote(method))
        #
        if page_tmplt == 'level3_tmplt':
            self.__depictLevel3WorkFlow()
        #
        self._dataInfo[page_tmplt] = [self.__dataD]
        self._dataInfo['entry_tmplt'] = [self.__lastInst]

    def DepictLevel2WorkFlow(self):
        """
        """
        self._reqObj.setValue("classID", "Annotate")
        for item in ('wf_inst_id', 'wf_class_id'):
            self._reqObj.setValue(item, self.__getInstanceData(item))
            #
        #
        depictWF = DepictWorkFlow(reqObj=self._reqObj, statusDB=self._statusDB, conFigObj=self._conFigObj, verbose=self._verbose, log=self._lfh)
        depictWF.getLevel2Setting()
        return depictWF.getPageText(page_id='workflow_tmplt')

    def __setup(self):
        """
        """
        self._connectStatusDB()
        self._getUserInfoDict()
        #
        dataD = self._statusDB.getDepInfo(depositionid=self._reqObj.getValue("identifier"))
        if dataD is None:
            return
        #
        pdbExtIdMap = self._getPdbExtIdMap([dataD])
        self.__dataD = processPublicIDs(dataD, pdbExtIdMap)

    def __getLastInst(self):
        """
        """
        dataD = self._statusDB.getLastInstance(depositionid=self._reqObj.getValue("identifier"))
        if dataD is None:
            return
        #
        pdbExtIdMap = self._getPdbExtIdMap([dataD])
        self.__lastInst = processPublicIDs(dataD, pdbExtIdMap)

    def __getInstanceData(self, item):
        """
        """
        val = ''
        if self.__lastInst and (item in self.__lastInst) and self.__lastInst[item]:
            val = str(self.__lastInst[item])
        #
        return val

    def __depictLevel3WorkFlow(self):
        """
        """
        logUtil = LogFileUtil(reqObj=self._reqObj, verbose=self._verbose, log=self._lfh)
        logFilePath = logUtil.getLogFile()
        if logFilePath:
            myD = {}
            for item in ('identifier', 'sessionid', 'instance', 'classID'):
                myD[item] = self._reqObj.getValue(item)
            #
            log_tmplt = self._getPageTemplate('download_wf_log_tmplt')
            self.__dataD['log_file'] = log_tmplt % myD
        #
        depictWF = DepictWorkFlow(reqObj=self._reqObj, statusDB=self._statusDB, conFigObj=self._conFigObj, verbose=self._verbose, log=self._lfh)
        wfD = depictWF.getLevel3Setting()
        self.__dataD.update(wfD)



##
# File:  ReadConFigFile.py
# Date:  16-Mar-2016
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

try:
    import cPickle as pickle
except ImportError:
    import pickle as pickle

import os
import sys
from wwpdb.io.file.mmCIFUtil import mmCIFUtil


class ReadConFigFile(object):
    """
    """
    def __init__(self, reqObj=None, configFile=None, verbose=False, log=sys.stderr):  # pylint: disable=unused-argument
        """
        """
        self.__reqObj = reqObj
        self.__configFile = configFile
        # self.__verbose = verbose
        # self.__lfh = log
        self.__topPath = self.__reqObj.getValue("TemplatePath")
        self.__configPath = os.path.join(self.__topPath, self.__configFile)
        #
        self.__sObj = self.__reqObj.newSessionObj()
        # self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        (base, _ext) = os.path.splitext(self.__configFile)
        self.__pickleFile = base + '.pkl'
        self.__picklePath = os.path.join(self.__sessionPath, self.__pickleFile)
        #

    def read(self):
        """
        """
        Dict = {}
        if os.access(self.__picklePath, os.F_OK):
            Dict = loadPickleFile(self.__sessionPath, self.__pickleFile)
            return Dict
        #
        cifObj = mmCIFUtil(filePath=self.__configPath)
        for item in ('user_template_mapping', 'user_tab_table_column_config', 'level1_template_definition', 'tab_definition_template',
                     'tab_preprocess_mapping', 'page_template', 'page_template_alias', 'page_template_parameter', 'function_parameter',
                     'ui_input_where_condition_binding', 'ui_input_dependence', 'table_data_field_binding'):
            vMap = {}
            try:
                vMap = getattr(self, "_read_%s" % item)(cifObj.GetValue(item))
            except:  # noqa: E722 pylint: disable=bare-except
                vMap = self.__read_list(cifObj.GetValue(item))
            #
            if vMap:
                Dict[item] = vMap
            #
        #
        tabDict = {}
        foundEmpty = False
        for item in ('tab_definition_table', 'table_definition', 'table_binding_definition', 'table_option_definition',
                     'table_column_definition', 'tab_table_sql_binding', 'sql_selection_definition'):
            try:
                tabDict[item] = getattr(self, "_read_%s" % item)(cifObj.GetValue(item))
            except:  # noqa: E722 pylint: disable=bare-except
                tabDict[item] = self.__read_list(cifObj.GetValue(item))
            #
            if not tabDict[item]:
                foundEmpty = True
            #
        #
        if tabDict and (not foundEmpty):
            Dict['table_definition'] = self.__processTableDefinition(tabDict)
        #
        dumpPickleFile(self.__sessionPath, self.__pickleFile, Dict)
        #
        return Dict

    def _read_user_template_mapping(self, vList):
        """
        """
        return self.__read_as_map(vList, 'user_id', 'template_id')

    def _read_user_tab_table_column_config(self, vList):
        """
        """
        Map = {}
        for vMap in vList:
            data = []
            for item in ('user_id', 'tab_id', 'table_id', 'data-field'):
                data.append(vMap[item])
            #
            Map['-'.join(data)] = vMap['data-visible']
        #
        return Map

    def _read_level1_template_definition(self, vList):
        """
        """
        Map = {}
        for vMap in vList:
            if not vMap['id'] in Map:
                Map[vMap['id']] = []
            #
            Map[vMap['id']].append(vMap)
        #
        return Map

    def _read_page_template(self, vList):
        """
        """
        return self.__read_as_Dictmap(vList)

    def _read_page_template_alias(self, vList):
        """
        """
        return self.__read_as_map(vList, 'id', 'page_id')

    def _read_page_template_parameter(self, vList):
        """
        """
        Map = {}
        for vMap in vList:
            if not vMap['page_id'] in Map:
                Map[vMap['page_id']] = [[], []]
            #
            if vMap['preprocess'].lower() == 'y':
                Map[vMap['page_id']][0].append(vMap)
            else:
                Map[vMap['page_id']][1].append(vMap)
            #
        #
        return Map

    def _read_table_option_definition(self, vList):
        """
        """
        return self.__read_as_map(vList, 'id', 'option')

    def _read_table_binding_definition(self, vList):
        """
        """
        return self.__read_as_Dictmap(vList)

    def _read_table_column_definition(self, vList):
        """
        """
        return self.__read_as_Dictmap(vList)

    def _read_tab_table_sql_binding(self, vList):
        """
        """
        Map = {}
        for vMap in vList:
            key = vMap['tab_id'] + '_' + vMap['table_id']
            Map[key] = vMap
        #
        return Map

    def _read_sql_selection_definition(self, vList):
        """
        """
        return self.__read_as_map(vList, 'id', 'sql')

    def _read_ui_input_where_condition_binding(self, vList):
        """
        """
        return self.__read_as_Dictmap(vList)

    def _read_ui_input_dependence(self, vList):
        """
        """
        return self.__read_as_Dictmap(vList)

    def _read_table_data_field_binding(self, vList):
        """
        """
        return self.__read_as_Dictmap(vList)

    def __read_as_Dictmap(self, vList):
        """
        """
        Map = {}
        for vMap in vList:
            Map[vMap['id']] = vMap
        #
        return Map

    def __read_as_map(self, vList, key_item, value_item):
        """
        """
        Map = {}
        for vMap in vList:
            Map[vMap[key_item]] = vMap[value_item]
        #
        return Map

    def __read_list(self, vList):
        """
        """
        Map = {}
        for vMap in vList:
            if not vMap['id'] in Map:
                Map[vMap['id']] = []
            #
            Map[vMap['id']].append(vMap)
        #
        return Map

    def __processTableDefinition(self, inDict):
        """ outDict = { 'tab_def_id': { 'table_id': { 'table_id': 'table_id', 'display': 'default_display', 'option': 'option_definition',
                        'load': 'default_load', 'column': [ {column_definition} ], 'sql_selection': 'sql_selection', 'sort_function':
                        'sort_function', 'sql_where_condition': 'sql_where_condition', 'sql_variable': 'sql_variable' } } }
        """
        outDict = {}
        for tab_id, vList in inDict['tab_definition_table'].items():
            tableList = {}
            for vDict in vList:
                option = ''
                if vDict['table_option_id'] in inDict['table_option_definition']:
                    option = inDict['table_option_definition'][vDict['table_option_id']]
                #
                tableDef = self.__getSqlInfo(tab_id + '_' + vDict['table_id'], inDict)
                column = self.__getTableColumns(vDict['table_definition_id'], inDict)
                if column:
                    if tableDef:
                        tableDef['column'] = column
                    #
                else:
                    tableDef = {}
                #
                bindingTableDef = {}
                if vDict['table_definition_id'] in inDict['table_binding_definition']:
                    bindingTableDef = inDict['table_binding_definition'][vDict['table_definition_id']]
                #
                if option and (tableDef or bindingTableDef):
                    tableDict = {}
                    tableDict['table_id'] = vDict['table_id']
                    tableDict['display'] = vDict['default_display']
                    tableDict['load'] = vDict['default_load']
                    if 'table_title' in vDict:
                        tableDict['table_title'] = vDict['table_title']
                    #
                    tableDict['option'] = option
                    if tableDef:
                        for k, v in tableDef.items():
                            tableDict[k] = v
                        #
                    else:
                        for item in ('binding_function', 'binding_class'):
                            if item in bindingTableDef:
                                tableDict[item] = bindingTableDef[item]
                            #
                        #
                    #
                    tableList[vDict['table_id']] = tableDict
                #
            #
            if tableList:
                outDict[tab_id] = tableList
            #
        #
        return outDict

    def __getTableColumns(self, table_definition_id, inDict):
        """
        """
        columnList = []
        if ('table_definition' not in inDict) or (table_definition_id not in inDict['table_definition']):
            return columnList
        #
        for vDict in inDict['table_definition'][table_definition_id]:
            if ('table_column_definition' not in inDict) or (vDict['column_definition'] not in inDict['table_column_definition']):
                return []
            #
            columnList.append(inDict['table_column_definition'][vDict['column_definition']])
        #
        return columnList

    def __getSqlInfo(self, key, inDict):
        """
        """
        myD = {}
        if ('tab_table_sql_binding' not in inDict) or (key not in inDict['tab_table_sql_binding']):
            return myD
        #
        # sql_selection = ''
        # sql_where_condition = ''
        # sql_variable = ''
        # sort_function = ''
        if ('select_definition_id' in inDict['tab_table_sql_binding'][key]) and inDict['tab_table_sql_binding'][key]['select_definition_id']:
            select_id = inDict['tab_table_sql_binding'][key]['select_definition_id']
            if ('sql_selection_definition' in inDict) and (select_id in inDict['sql_selection_definition']) and inDict['sql_selection_definition'][select_id]:
                myD['sql_selection'] = inDict['sql_selection_definition'][select_id]
            #
        #
        if not myD:
            return myD
        #
        for options in (('sql_where_condition', 'where_condition'), ('order_condition', 'additional_order_condition'),
                        ('sql_variable', 'variable'), ('sort_function', 'additional_sort_function')):
            if (not options[1] in inDict['tab_table_sql_binding'][key]) or (not inDict['tab_table_sql_binding'][key][options[1]]):
                continue
            #
            myD[options[0]] = inDict['tab_table_sql_binding'][key][options[1]]
        #
        return myD


def dumpPickleFile(sessionPath, filename, data):
    """
    """
    pickle_file = os.path.join(sessionPath, filename)
    f = open(pickle_file, 'wb')
    pickle.dump(data, f)
    f.close()


def loadPickleFile(sessionPath, filename):
    """
    """
    pickle_file = os.path.join(sessionPath, filename)
    if not os.access(pickle_file, os.F_OK):
        return None
    #
    f = open(pickle_file, 'rb')
    data = pickle.load(f)
    f.close()
    return data



##
# File:  ServerInfoUtil.py
# Date:  17-Mar-2016
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import sys

from wwpdb.utils.wf.dbapi.WFEtime import getTimeNow
from wwpdb.apps.workmanager.depict.DepictBase import DepictBase


class ServerInfoUtil(DepictBase):
    """
    """
    def __init__(self, reqObj=None, statusDB=None, conFigObj=None, verbose=False, log=sys.stderr):
        """
        """
        super(ServerInfoUtil, self).__init__(reqObj=reqObj, statusDB=statusDB, conFigObj=conFigObj, verbose=verbose, log=log)
        #

    def getServerInfo(self):
        """
        """
        text = ''
        timeNow = getTimeNow()
        self._connectStatusDB()
        if self._statusDB:
            serverList = self._statusDB.getServerInfo()
            for server in serverList:
                dataD = {}
                dataD['host'] = server['hostname']
                dif = (round(float(timeNow) - float(server['status_timestamp']))) / 10.0
                dataD['time'] = str(dif)
                if dif < 1.1:
                    dataD['comment'] = 'Good'
                elif dif < 2.1:
                    dataD['comment'] = 'OK'
                elif dif < 5.1:
                    dataD['comment'] = 'Poor'
                else:
                    dataD['comment'] = 'Unavailable'
                #
                self._dataInfo['server_info_tmplt'] = [dataD]
                text += self.getPageText(page_id='server_info_tmplt')
            #
        #
        return text



##
# File:  LoadRemindMessageTrack.py
# Date:  27-April-2016
# Updates: 31-October-2025 - Refactored to use msgmodule DataAccessLayer instead of CIF file parsing
##
"""
API for loading message receiving/sending information into status database.

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"  # TODO: Update version after refactor  # pylint: disable=fixme

import datetime
import getopt
import os
import sys
import time
import traceback

import MySQLdb
from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.utils.wf.dbapi.DbConnection import DbConnection

# Import msgmodule utilities - ExtractMessage does the heavy lifting
from wwpdb.apps.msgmodule.util.ExtractMessage import ExtractMessage


class DbApiUtil(object):
    """ Class for making status database connection
    """

    def __init__(self, siteId=None, verbose=False, log=sys.stderr):  # pylint: disable=unused-argument
        """ Initialization
        """
        self.__siteId = siteId
        # self.__verbose = verbose
        self.__lfh = log
        self.__cI = ConfigInfo(self.__siteId)
        self.__myDb = DbConnection(dbServer=self.__cI.get("SITE_DB_SERVER"),
                                   dbHost=self.__cI.get("SITE_DB_HOST_NAME"),
                                   dbName=self.__cI.get("SITE_DB_DATABASE_NAME"),
                                   dbUser=self.__cI.get("SITE_DB_USER_NAME"),
                                   dbPw=self.__cI.get("SITE_DB_PASSWORD"),
                                   dbPort=int(self.__cI.get("SITE_DB_PORT_NUMBER")),
                                   dbSocket=self.__cI.get("SITE_DB_SOCKET"))
        self.__dbcon = self.__myDb.connect()
        self.__Nretry = 5
        self.__dbState = 0

    def runUpdate(self, table=None, where=None, data=None):
        """ Insertion/Update table based on table name, where condition(s) and data content(s)
        """
        if not table:
            return None
        #
        if (not where) and (not data):
            return None
        #
        rowExists = False
        if where:
            sql = "select * from " + str(table) + " where " + ' and '.join(
                ["%s = '%s'" % (k, v.replace("'", "\\'")) for k, v in where.items()])
            rows = self.runSelectSQL(sql)
            if rows and len(rows) > 0:
                rowExists = True
            #
        #
        if rowExists and (not data):
            return 'OK'
        #
        if rowExists:
            sql = "update " + str(table) + " set " + ','.join(
                ["%s = '%s'" % (k, v.replace("'", "\\'")) for k, v in data.items()])
            if "major_issue" not in data:
                sql += ',major_issue = NULL'
            #
            if where:
                sql += ' where ' + ' and '.join(["%s = '%s'" % (k, v.replace("'", "\\'")) for k, v in where.items()])
            #
        else:
            sql = "insert into " + str(table) + " (" + ','.join(['%s' % (k) for k, v in where.items()])
            if data:
                sql += "," + ','.join(['%s' % (k) for k, v in data.items()])
            #
            sql += ") values (" + ','.join(["'%s'" % (v.replace("'", "\\'")) for k, v in where.items()])
            if data:
                sql += "," + ','.join(["'%s'" % (v.replace("'", "\\'")) for k, v in data.items()])
            #
            sql += ")"
        #
        return self.runUpdateSQL(sql)

    def runSelectSQL(self, sql):
        """ Select table row(s) based on sql command
        """
        for retry in range(1, self.__Nretry):
            ret = self.__runSelectSQL(sql)
            if ret is None:
                if self.__dbState > 0:
                    time.sleep(retry * 2)
                    if not self.__reConnect():
                        return None
                else:
                    return None
                #
            else:
                return ret
            #
        #
        return None

    def runUpdateSQL(self, sql):
        """ Insertion/Update table based on sql command
        """
        for retry in range(1, self.__Nretry):
            ret = self.__runUpdateSQL(sql)
            if ret is None:
                if self.__dbState > 0:
                    time.sleep(retry * 2)
                    if not self.__reConnect():
                        return None
                else:
                    return None
                #
            else:
                return ret
            #
        #
        return None

    def __runSelectSQL(self, query):
        """ Execute selection query command
        """
        rows = ()
        try:
            self.__dbcon.commit()
            curs = self.__dbcon.cursor(MySQLdb.cursors.DictCursor)
            curs.execute(query)
            rows = curs.fetchall()
        except MySQLdb.Error as e:
            self.__dbState = e.args[0]
            self.__lfh.write("Database error %d: %s\n" % (e.args[0], e.args[1]))

        return rows

    def __runUpdateSQL(self, query):
        """ Execute insertion/update query command
        """
        try:
            curs = self.__dbcon.cursor()
            curs.execute("set autocommit=0")
            _nrows = curs.execute(query)  # noqa: F841
            self.__dbcon.commit()
            curs.execute("set autocommit=1")
            curs.close()
            return 'OK'
        except MySQLdb.Error as e:
            self.__dbcon.rollback()
            self.__dbState = e.args[0]
            self.__lfh.write("Database error %d: %s\n" % (e.args[0], e.args[1]))
        #
        return None

    def __reConnect(self):
        """ Make database re-connection
        """
        try:
            self.__myDb.close(self.__dbcon)
        except MySQLdb.Error:
            self.__lfh.write("+DbApiUtil.reConnect() DB connection lost - cannot close\n")
            self.__lfh.write("+DbApiUtil.reConnect() Re-connecting to the database ..\n")
            self.__lfh.write("+DbApiUtil.reConnect() UTC time = %s\n" % datetime.datetime.utcnow())
        #
        for i in range(1, self.__Nretry):
            try:
                self.__dbcon = self.__myDb.connect()
                self.__dbState = 0
                return True
            except MySQLdb.Error:
                self.__lfh.write("+DbApiUtil.reConnect() Cannot get re-connection : trying again\n")
                time.sleep(2 * i)
            #
        #
        return False


class LoadRemindMessageTrack(object):
    """ Class for loading message receiving/sending information into status database
    """

    def __init__(self, siteId=None, verbose=False, log=sys.stderr):
        """ Initialization
        """
        self.__siteId = siteId
        self.__verbose = verbose
        self.__lfh = log
        self.__statusDB = DbApiUtil(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)

        # Use ExtractMessage which automatically handles legacy vs modern communication
        self.__extractMessage = ExtractMessage(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)

    def UpdateBasedIDList(self, depIDList):
        """ Update remind_message_track table based depID list ( comma separate )
        """
        for depID in depIDList.split(','):
            depID = depID.strip()
            self.__updateEntry(depID)
        #

    def UpdateBasedInputIDfromFile(self, filename):
        """ Update remind_message_track table based depID list from file (assume each ID per line)
        """
        if (not os.access(filename, os.F_OK)):
            return
        #
        f = open(filename, 'r')
        data = f.read()
        f.close()
        #
        for depID in data.split('\n'):
            depID = depID.strip()
            self.__updateEntry(depID)
        #

    def __updateEntry(self, depID):
        """ Get remind_message_track information and update table
        """
        if not depID:
            return
        #
        trackMap = self.__getRemindMessageTrack(depID)
        if not trackMap:
            return
        #
        self.__statusDB.runUpdate(table='remind_message_track', where={'dep_set_id': depID}, data=trackMap)

    def __getRemindMessageTrack(self, depID):
        """ Get remind_message_track table information for given depID
        """
        return self.__getRemindMessageTrackFromExtractMessage(depID)

    def __getRemindMessageTrackFromExtractMessage(self, depID):
        """ Get remind_message_track table information using ExtractMessage utility """
        # Use ExtractMessage API methods - let it handle the heavy lifting
        trackMap = {}

        try:
            # Get last message received date from depositor
            last_received = self.__extractMessage.getLastReceivedMsgDatetime(depID)
            if last_received:
                trackMap['last_message_received_date'] = last_received.strftime('%Y-%m-%d')

            # Get last message sent to depositor
            last_sent = self.__extractMessage.getLastSentMsgDatetime(depID)
            if last_sent:
                trackMap['last_message_sent_date'] = last_sent.strftime('%Y-%m-%d')

            # Get last manual reminder sent to depositor
            last_reminder = self.__extractMessage.getLastManualReminderDatetime(depID)
            if last_reminder:
                trackMap['last_reminder_sent_date'] = last_reminder.strftime('%Y-%m-%d')

            # Get last validation report info (returns tuple of datetime and major_issue boolean)
            validation_info = self.__extractMessage.getLastValidation(depID)
            if validation_info[0]:  # validation_info is (datetime, major_issue_boolean)
                trackMap['last_validation_sent_date'] = validation_info[0].strftime('%Y-%m-%d')
                if validation_info[1]:  # major_issue_boolean
                    trackMap['major_issue'] = 'Yes'

        except Exception as e:
            if self.__verbose:
                self.__lfh.write("Error getting message track from ExtractMessage for %s: %s\n" % (depID, str(e)))

        return trackMap


def load_main():
    try:
        opts, _args = getopt.getopt(sys.argv[1:], "i:f:")
        idlist = ''
        filename = ''
        for opt, arg in opts:
            if opt in ("-i"):
                idlist = arg
            elif opt in ("-f"):
                filename = arg
            #
        #
        if idlist or filename:
            siteId = str(os.getenv('WWPDB_SITE_ID'))
            api = LoadRemindMessageTrack(siteId=siteId, verbose=False, log=sys.stderr)
            if idlist:
                api.UpdateBasedIDList(idlist)
            #
            if filename:
                api.UpdateBasedInputIDfromFile(filename)
            #
        #
    except:  # noqa: E722 pylint: disable=bare-except
        traceback.print_exc(file=sys.stderr)
    #


if __name__ == '__main__':
    load_main()



##
# File:  SearchUtil.py
# Date:  04-April-2016
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import sys

from wwpdb.apps.workmanager.db_access.ContentDbApi import ContentDbApi
from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi
from wwpdb.apps.workmanager.depict.ReadConFigFile import ReadConFigFile, dumpPickleFile, loadPickleFile


class SearchUtil(object):
    """
    """
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
        """
        self.__reqObj = reqObj
        self.__verbose = verbose
        self.__lfh = log
        self.__siteId = self.__reqObj.getValue("WWPDB_SITE_ID")
        # self.__topPath = self.__reqObj.getValue("TemplatePath")
        #
        self.__statusDB = None
        self.__contentDB = None
        self.__conFigObj = None
        #
        self.__sObj = self.__reqObj.newSessionObj()
        # self.__sessionId = self.__sObj.getId()
        self.__sessionPath = self.__sObj.getPath()
        #

    def updateSql(self):
        """
        """
        index = self.__reqObj.getValue('index')
        search_type = self.__reqObj.getValue('search_type')
        value = self.__reqObj.getValue('value')
        if (not index) or (not search_type):
            return
        #
        tableContentMap = loadPickleFile(self.__sessionPath, 'TableContentMap.pkl')
        if (not tableContentMap) or (index not in tableContentMap):
            return
        #
        self.__readConFigObj()
        if (not self.__conFigObj) or ('ui_input_where_condition_binding' not in self.__conFigObj) or \
           (search_type not in self.__conFigObj['ui_input_where_condition_binding']):
            return
        #
        myD = {}
        if ('dependence_id' in self.__conFigObj['ui_input_where_condition_binding'][search_type]) and \
           self.__conFigObj['ui_input_where_condition_binding'][search_type]['dependence_id']:
            myD['value'] = self.__processDependence(self.__conFigObj['ui_input_where_condition_binding'][search_type]['dependence_id'], value)
        elif search_type == 'entry_by_ids':
            self.__connectStatusDB()
            _error_message, entryIdList = self.__statusDB.getEntryIdListFromInputIdString(value)
            myD['value'] = "', '".join(entryIdList)
        elif (search_type == 'user_by_ids') or (search_type == 'dep_by_ids') or (search_type == 'group_by_ids'):
            myList = []
            for ID in value.strip().split(','):
                myList.append(ID.strip())
            #
            myD['value'] = "', '".join(myList)
        else:
            myD['value'] = value
        #
        where_condition = self.__conFigObj['ui_input_where_condition_binding'][search_type]['where_condition'] % myD
        tableContentMap[index]['sql'] = tableContentMap[index]['sql_selection'] + ' ' + where_condition
        dumpPickleFile(self.__sessionPath, 'TableContentMap.pkl', tableContentMap)

    def __readConFigObj(self):
        """
        """
        readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='level1_config.cif', verbose=self.__verbose, log=self.__lfh)
        self.__conFigObj = readUtil.read()

    def __processDependence(self, dependence_id, value):
        """
        """
        if (not self.__conFigObj) or ('ui_input_dependence' not in self.__conFigObj) or (dependence_id not in self.__conFigObj['ui_input_dependence']):
            return value
        #
        sql = self.__conFigObj['ui_input_dependence'][dependence_id]['sql']
        if value:
            myD = {}
            myD['value'] = value
            sql = sql % myD
        #
        retList = []
        if self.__conFigObj['ui_input_dependence'][dependence_id]['db'] == 'contentDB':
            self.__connectContentDB()
            retList = self.__contentDB.runSelectSQL(sql)
        else:
            self.__connectStatusDB()
            retList = self.__statusDB.runSelectSQL(sql)
        #
        idList = []
        if retList:
            for dataD in retList:
                idList.append(dataD['id'])
            #
        #
        return "', '".join(idList)

    def __connectStatusDB(self):
        """
        """
        if not self.__statusDB:
            self.__statusDB = StatusDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        #

    def __connectContentDB(self):
        """
        """
        if not self.__contentDB:
            self.__contentDB = ContentDbApi(siteId=self.__siteId, verbose=self.__verbose, log=self.__lfh)
        #



##
# File:  DepictBase.py
# Date:  17-Mar-2016
#
# Updates:
#  09-Dec-2024  zf   add _getPdbExtIdMap() method and 'ext_pdb_id'
#
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


try:
    import builtins
except ImportError:
    import __builtin__ as builtins
import os
import sys

from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.apps.workmanager.db_access.ContentDbApi import ContentDbApi
from wwpdb.apps.workmanager.db_access.StatsUtil import StatsUtil
from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi


class DepictBase(object):
    """
    """
    def __init__(self, reqObj=None, statusDB=None, conFigObj=None, verbose=False, log=sys.stderr):
        """
        """
        self._reqObj = reqObj
        self._conFigObj = conFigObj
        self._verbose = verbose
        self._lfh = log
        self._siteId = self._reqObj.getValue("WWPDB_SITE_ID")
        self._topPath = self._reqObj.getValue("TemplatePath")
        #
        self.__urlMap = {}
        self._UtilClass = {}
        self._userInfo = {}
        self._dataInfo = {}
        self._uInfoFlag = False
        self._statusDB = statusDB
        self._contentDB = None
        self.__getUrlMap()

    def getPageText(self, page_id='', paraD=None):
        """
        """
        if paraD is None:
            paraD = {}
        page_template, alias_page_id = self.__getPageTemplateAndAliasID(page_id)
        if not page_template:
            return ''
        #
        if 'page_template_parameter' not in self._conFigObj:
            return ''
        #
        pageD = self._conFigObj['page_template'][alias_page_id]
        #       if ('dependence' in pageD) and (not pageD['dependence'] in self._dataInfo):
        #           return ''
        #       #
        if paraD:
            page_template = self._preProcessParameterMap(page_template, paraD)
        #
        page_parameter = self._conFigObj['page_template_parameter'][alias_page_id]
        page_template = self.__preProcessParameter(page_id, alias_page_id, page_template, page_parameter[0])
        #
        return_text = ''
        if ('repeat' in pageD) and (pageD['repeat'] == 'yes') and (page_id in self._dataInfo):
            while len(self._dataInfo[page_id]) > 0:
                text = self.__processParameter(page_id, alias_page_id, page_template, page_parameter[1])
                if return_text:
                    return_text += '\n'
                return_text += text
                self._dataInfo[page_id].pop(0)
            #
        else:
            return_text = self.__processParameter(page_id, alias_page_id, page_template, page_parameter[1])
        #
        return return_text

    def getDataResult(self, input_type='', return_type='string', delimiter=','):
        """
        """
        if delimiter == 'newline':
            delimiter = '\n'
        elif delimiter == 'empty':
            delimiter = ''
        #
        if input_type and (input_type in self._dataInfo):
            return self._processResult(self._dataInfo[input_type], return_type, delimiter)
        #
        if return_type == 'dict':
            return {}
        elif return_type == 'list':
            return []
        else:
            return ''

    def _getPageTemplate(self, page_id):
        """
        """
        if ('page_template' not in self._conFigObj) or (not page_id) or (page_id not in self._conFigObj['page_template']):
            return ''
        #
        pageD = self._conFigObj['page_template'][page_id]
        page_template = pageD['page']
        if pageD['type'] == 'file':
            page_template = self.__readTemplate(pageD['page'])
        #
        return page_template

    def _preProcessParameterMap(self, template, myD):
        """
        """
        for k, v in myD.items():
            template = template.replace('%(' + k + ')s', v)
        #
        return template

    def _getUserInfo(self, parameter):
        """
        """
        self._getUserInfoDict()
        #
        val = ''
        if self._userInfo and (parameter in self._userInfo):
            val = self._userInfo[parameter]
        #
        return val

    def _getUserInfoDict(self):
        """
        """
        if self._uInfoFlag:
            return
        #
        self._connectStatusDB()
        #
        if self._statusDB:
            if self._reqObj.getValue('username') and self._reqObj.getValue('password'):
                self._userInfo = self._statusDB.Autenticate(self._reqObj.getValue('username'), self._reqObj.getValue('password'))
            #
            if not self._userInfo and self._reqObj.getValue('annotator'):
                self._userInfo = self._statusDB.getUserByInitial(initial=self._reqObj.getValue('annotator'))
            #
            if not self._userInfo and self._reqObj.getValue('initials'):
                self._userInfo = self._statusDB.getUserByInitial(initial=self._reqObj.getValue('initials'))
            #
        #
        self._uInfoFlag = True

    def _get_entry_by_initial_enum_list(self):
        """
        """
        self._getUserInfoDict()
        rows = self._statusDB.getAnnUser(self._userInfo['site'])
        rlist = []
        for row in rows:
            rlist.append([str(row['initials']), str(row['first_name']) + ' ' + str(row['last_name'])])
        #
        rlist.sort(key=lambda data: data[1])
        rlist.insert(0, ['', ''])
        return rlist

    def _getAnnotatorSelection(self):
        """
        """
        AnnotatorInfo = self._get_entry_by_initial_enum_list()
        return self.__getSelectionOption(AnnotatorInfo)

    def _getPrivilegeTableRows(self):
        """
        """
        rows = self.__getSiteUserInfo()
        if not rows:
            return
        #
        table_row1 = []
        table_row2 = []
        for row in rows:
            if str(row['active']) != '0':
                continue
            #
            row['checked'] = ''
            row['disabled'] = ''
            if row['user_name'] == self._userInfo['user_name']:
                row['disabled'] = 'disabled="true"'
            #
            if str(row['code']).upper() == 'ANN':
                row['value'] = 'LANN'
                table_row1.append(row)
            elif str(row['code']).upper() == 'LANN':
                row['value'] = 'ANN'
                table_row2.append(row)
            #
        #
        self._dataInfo['privilege_ann_table_row_tmplt'] = table_row1
        self._dataInfo['privilege_lead_table_row_tmplt'] = table_row2

    def _getGroupSelection(self):
        """
        """
        GroupInfo = []
        GroupInfo.append(['', ''])
        self._getUserInfoDict()
        if self._statusDB:
            rows = self._statusDB.getSiteGroup(self._userInfo['site'])
            for row in rows:
                GroupInfo.append([str(row['group_id']), str(row['group_name'])])
            #
        #
        return self.__getSelectionOption(GroupInfo)

    def _getActiveTableRow(self):
        """
        """
        rows = self.__getSiteUserInfo()
        if not rows:
            return
        #
        for row in rows:
            row['checked'] = ''
            row['value'] = str(row['active'])
            if str(row['active']) == '0':
                row['checked'] = 'checked'
            #
            row['disabled'] = ''
            if row['user_name'] == self._userInfo['user_name']:
                row['disabled'] = 'disabled="true"'
            #
        #
        self._dataInfo['atvice_user_table_row_tmplt'] = list(rows)

    def _connectStatusDB(self):
        """
        """
        if not self._statusDB:
            self._statusDB = StatusDbApi(siteId=self._siteId, verbose=self._verbose, log=self._lfh)
        #

    def _connectContentDB(self):
        """
        """
        if not self._contentDB:
            self._contentDB = ContentDbApi(siteId=self._siteId, verbose=self._verbose, log=self._lfh)
        #

    def _processResult(self, resultObj, return_type, delimiter):
        """
        """
        stringFlag = False
        if return_type == 'string':
            stringFlag = True
        #
        if isinstance(resultObj, dict):
            return self.__writeDirValue(resultObj, delimiter, stringFlag)
        elif isinstance(resultObj, list):
            return self.__writeListValue(resultObj, delimiter, stringFlag)
        else:
            return str(resultObj)
        #

    def _processBindingClass(self, bindingClass):
        """
        """
        if bindingClass in self._UtilClass:
            return
        #
        if bindingClass == 'StatsUtil':
            self._UtilClass[bindingClass] = StatsUtil(siteId=self._siteId, verbose=self._verbose, log=self._lfh)
        #

    def _processBaseUrl(self, class_id):
        """
        """
        if class_id in self.__urlMap:
            return self.__urlMap[class_id]
        #
        return ''

    def _getPdbExtIdMap(self, dataDictList):
        """
        """
        self._connectContentDB()
        #
        pdbIdList = []
        for dataDict in dataDictList:
            if ('pdb_id' in dataDict) and dataDict['pdb_id']:
                pdbIdList.append(dataDict['pdb_id'])
            #
        #
        return self._contentDB.getPdbExtIdMap(pdbIdList)

    def __getUrlMap(self):
        """
        """
        mod_url_list = [['AnnMod', 'SITE_ANN_TASKS_URL'], ['AnnModUI', 'SITE_ANN_TASKS_URL'],
                        ['LigMod', 'SITE_LE_URL'], ['LigModUI', 'SITE_LE_URL'],
                        ['SeqMod', 'SITE_SE_URL'], ['SeqModUI', 'SITE_SE_URL'],
                        ['TransMod', 'SITE_TRANS_EDITOR_URL'], ['TransModUI', 'SITE_TRANS_EDITOR_URL'],
                        ['ValMod', 'SITE_VAL_TASKS_URL'], ['ValModUI', 'SITE_VAL_TASKS_URL']]
        #
        cI = ConfigInfo(self._siteId)
        for mlist in mod_url_list:
            self.__urlMap[mlist[0]] = cI.get(mlist[1])
        #

    def __getPageTemplateAndAliasID(self, page_id):
        """
        """
        alias_page_id = page_id
        page_template = self._getPageTemplate(page_id)
        if page_template:
            return page_template, alias_page_id
        #
        if ('page_template_alias' in self._conFigObj) and (page_id in self._conFigObj['page_template_alias']):
            tmp_page_id = self._conFigObj['page_template_alias'][page_id]
            tmp_page_template = self._getPageTemplate(tmp_page_id)
            if tmp_page_template:
                page_template = tmp_page_template
                alias_page_id = tmp_page_id
            #
        #
        return page_template, alias_page_id

    def __preProcessParameter(self, page_id, alias_page_id, template, paraList):
        """
        """
        myD = self.__getParameter(page_id, alias_page_id, paraList)
        if not myD:
            return template
        #
        return self._preProcessParameterMap(template, myD)

    def __processParameter(self, page_id, alias_page_id, template, paraList):
        """
        """
        myD = self.__getParameter(page_id, alias_page_id, paraList)
        if not myD:
            return template
        #
        return template % myD

    def __getParameter(self, page_id, alias_page_id, paraList):
        """
        """
        myD = {}
        if not paraList:
            return myD
        #
        for paraMap in paraList:
            myD[paraMap['variable']] = ''
            if paraMap['type'] == 'page_template':
                myD[paraMap['variable']] = self.getPageText(page_id=paraMap['value'])
            elif paraMap['type'] == 'function':
                myD[paraMap['variable']] = self.__callFunc(alias_page_id, paraMap['variable'], paraMap['value'], myD)
            elif paraMap['type'] == 'sessionInfo':
                myD[paraMap['variable']] = self._reqObj.getValue(paraMap['value'])
            elif paraMap['type'] == 'userInfo':
                myD[paraMap['variable']] = self._getUserInfo(paraMap['value'])
            elif paraMap['type'] == 'dataInfo':
                if (page_id in self._dataInfo) and self._dataInfo[page_id]:
                    if paraMap['value'] in self._dataInfo[page_id][0]:
                        myD[paraMap['variable']] = self._dataInfo[page_id][0][paraMap['value']]
                    #
                elif ('data_for_all' in self._dataInfo) and self._dataInfo['data_for_all']:
                    if paraMap['value'] in self._dataInfo['data_for_all'][0]:
                        myD[paraMap['variable']] = self._dataInfo['data_for_all'][0][paraMap['value']]
                    #
                #
            elif paraMap['type'] == 'constant':
                myD[paraMap['variable']] = paraMap['value']
            #
        #
        return myD

    def __callFunc(self, page_id, variable, funcDef, myD):
        """
        """
        val = ''
        funcDef = funcDef.replace(' ', '')
        paraD = self.__getFuncParameter(page_id, variable, myD)
        if not paraD:
            paraD = self.__getFuncParameter(page_id, funcDef, myD)
        #
        if not paraD:
            paraD = self.__getFuncParameter(variable, funcDef, myD)
        #
        spList = funcDef.split(',')
        #
        if spList[0] == "self":
            if paraD:
                val = getattr(self, "%s" % spList[1])(**paraD)
            else:
                val = getattr(self, "%s" % spList[1])()
            #
        elif spList[0] in self._UtilClass:
            if paraD:
                val = getattr(self._UtilClass[spList[0]], "%s" % spList[1])(**paraD)
            else:
                val = getattr(self._UtilClass[spList[0]], "%s" % spList[1])()
            #
        #
        return val

    def __getFuncParameter(self, page_id, funcDef, myD):  # pylint: disable=unused-argument
        """
        """
        paraD = {}
        key = page_id + ',' + funcDef
        #
        if ('function_parameter' not in self._conFigObj) or (key not in self._conFigObj['function_parameter']):
            return paraD
        #
        for paraMap in self._conFigObj['function_parameter'][key]:
            if ('value_type' in paraMap) and ('parameter_type' in paraMap) and paraMap['parameter_type'] == 'constant':
                paraD[paraMap['name']] = getattr(builtins, '%s' % paraMap['value_type'])(paraMap['value'])
            else:
                paraD[paraMap['name']] = paraMap['value']
            #
        #
        return paraD

    def __readTemplate(self, template_file):
        """
        """
        fPath = os.path.join(self._topPath, template_file)
        ifh = open(fPath, 'r')
        sIn = ifh.read()
        ifh.close()
        return sIn

    def __writeDirValue(self, obj, delimiter, stringFlag):
        """
        """
        quotation = "'"
        if stringFlag:
            quotation = ""
        #
        text = ''
        for k, v in obj.items():
            if text:
                text += delimiter
            #
            if isinstance(v, dict):
                text += quotation + str(k) + quotation + ":" + self.__writeDirValue(v, delimiter, stringFlag)
            elif isinstance(v, list):
                text += quotation + str(k) + quotation + ":" + self.__writeListValue(v, delimiter, stringFlag)
            else:
                text += quotation + str(k) + quotation + ":" + quotation + str(v) + quotation
            #
        #
        if stringFlag:
            return text
        #
        text = text.replace(quotation + "False" + quotation, "false").replace(quotation + "True" + quotation, "true")
        text = text.replace(quotation + "false" + quotation, "false").replace(quotation + "true" + quotation, "true")
        return '{' + text + '}'

    def __writeListValue(self, obj, delimiter, stringFlag):
        """
        """
        quotation = "'"
        if stringFlag:
            quotation = ""
        #
        text = ''
        for v in obj:
            if text:
                text += delimiter
            #
            if isinstance(v, dict):
                text += self.__writeDirValue(v, delimiter, stringFlag)
            elif isinstance(v, list):
                text += self.__writeListValue(v, delimiter, stringFlag)
            else:
                text += quotation + str(v) + quotation
            #
        #
        if stringFlag:
            return text
        #
        # text = text.replace("'False'", "false").replace("'True'", "true")
        text = text.replace(quotation + "False" + quotation, "false").replace(quotation + "True" + quotation, "true")
        text = text.replace(quotation + "false" + quotation, "false").replace(quotation + "true" + quotation, "true")
        return '[' + text + ']'

    def __getSelectionOption(self, SelectionInfo):
        """
        """
        SelectionHtml = ''
        for slist in SelectionInfo:
            SelectionHtml += '<option value="' + slist[0] + '"'
            if slist[0] == '':
                SelectionHtml += ' selected'
            #
            SelectionHtml += '>' + slist[1] + '</option>'
        #
        return SelectionHtml

    def __getSiteUserInfo(self):
        """
        """
        self._getUserInfoDict()
        if not self._statusDB:
            return []
        #
        return self._statusDB.getSiteUser(self._userInfo['site'])


def processPublicIDs(dataDict, pdbExtIdMap):
    if not dataDict:
        dataDict = {}
        dataDict['ext_pdb_id'] = '-'
        dataDict['pdb_id'] = '-'
        dataDict['bmrb_id'] = '-'
        dataDict['emdb_id'] = '-'
    #
    for item in ('pdb_id', 'bmrb_id', 'emdb_id'):
        if (item not in dataDict) or (not dataDict[item]) or (dataDict[item] == '?'):
            dataDict[item] = '-'
        #
        if item == 'pdb_id':
            if (dataDict['pdb_id'] in pdbExtIdMap) and pdbExtIdMap[dataDict['pdb_id']]:
                dataDict['ext_pdb_id'] = pdbExtIdMap[dataDict['pdb_id']]
            else:
                dataDict['ext_pdb_id'] = dataDict['pdb_id']
            #
        #
    #
    return dataDict



##
# File:  ContentDbApi.py
# Date:  15-May-2015
#
# Updates:
#  09-Dec-2024  zf   add getPdbExtIdMap() method.
#
##
"""
Providing addintaional APIs for WFE to get info from db_internal database.

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import sys

from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.apps.workmanager.db_access.DbApiUtil import DbApiUtil


class ContentDbApi(object):
    __schemaMap = { "AUDIT_HISTORY" : "select h.Structure_ID, h.ordinal, h.major_revision, h.minor_revision, h.revision_date, h.internal_version, d.description from pdbx_audit_revision_history as h " +
                         "LEFT JOIN pdbx_audit_revision_details d ON h.Structure_ID = d.Structure_ID and h.ordinal = d.revision_ordinal and d.type='Coordinate replacement' " +
                         "where h.Structure_ID = '%s' order by ordinal",
                    "CONTACT_AUTHOR" : "select Structure_ID as id, email, name_first, name_mi, name_last, role, country from pdbx_contact_author where Structure_ID = '%s'",
                    "CONTACT_AUTHOR_LIST" : "select Structure_ID as id, email, name_first, name_mi, name_last, role, country from pdbx_contact_author where Structure_ID in ( '%s' )",
                    "CONTACT_AUTHOR_PI" : "select Structure_ID as id, email, name_first, name_mi, name_last, role, country from pdbx_contact_author where Structure_ID = '%s' " +
                                          "and role = 'principal investigator/group leader'",
                    "CONTACT_AUTHOR_PI_LIST" : "select Structure_ID as id, email, name_first, name_mi, name_last, role, country from pdbx_contact_author where Structure_ID in ( '%s' ) " +
                                               "and role = 'principal investigator/group leader'",
                    "GET_REMINDER_LIST" : "select structure_id from rcsb_status where status_code in ('WAIT','PROC','REPL','AUTH','AUCO') and " +
                                          "rcsb_annotator = '%s' and initial_deposition_date <= DATE_SUB(curdate(), interval 21 day) order by structure_id",
                    "GET_DAILY_STATS" : "select rcsb_annotator from rcsb_status where status_code not in ('PROC','WAIT','POLC','AUCO') " +
                                        "and date_begin_processing  = '%s'",
                    "GET_RANGE_STATS" : "select rcsb_annotator from rcsb_status where status_code not in ('PROC','WAIT','POLC','AUCO') " +
                                        "and date_begin_processing  >= '%s' and date_begin_processing <= '%s'",
                "GET_INPROCESS_STATS" : "select rcsb_annotator, status_code from rcsb_status where status_code in ('WAIT','PROC','AUTH','POLC','REPL')",
                   "GET_RELEASE_DATE" : "select structure_id, pdb_id, date_of_RCSB_release from rcsb_status where structure_id in ( '%s' ) order by structure_id",
                "GET_EM_RELEASE_DATE" : "select structure_id, current_status, map_release_date date_of_EM_release from em_admin where structure_id in ( '%s' ) order by structure_id",
                 "GET_REPLACE_COUNTS" : "select s2.name, s2.identifier_ORCID, sum(s2.count) as numreplace from " +
                        "(select c.identifier_ORCID, CONCAT('', c.name_last, ', ', c.name_first) as name, s1.count from pdbx_contact_author as c, " + 
                        "(select d.Structure_id, count(d.Structure_id) as count   from pdbx_audit_revision_details as d, pdbx_audit_revision_history as h where " +
                        "h.Structure_id = d.Structure_id and h.ordinal = d.revision_ordinal and d.type='Coordinate replacement' and DATEDIFF(CURDATE(), h.revision_date) <= 365 " +
                        "group by d.Structure_id  ) s1  where s1.Structure_id = c.Structure_Id and c.role='principal investigator/group leader' " +
                        "order by c.identifier_ORCID ) s2 group by identifier_ORCID, name order by name",
                 "GET_LIGAND_ID_LIST" : "select Structure_ID, comp_id from pdbx_entity_nonpoly where Structure_ID in ( '%s' )",
                 "GET_EXT_PDB_ID_INFO": "select distinct database_code,pdbx_database_accession from database_2 where database_id = 'PDB' and pdbx_database_accession " +
                                        "is not NULL and pdbx_database_accession != '' and database_code in ( '%s' )"
                   }
    """
    """
    def __init__(self, siteId=None, verbose=False, log=sys.stderr):
        """
        """
        self.__lfh = log
        self.__verbose = verbose
        self.__siteId = siteId
        self.__cI = ConfigInfo(self.__siteId)
        self.__dbServer = self.__cI.get("SITE_DB_SERVER")
        self.__dbHost = self.__cI.get("SITE_DB_HOST_NAME")
        self.__dbName = "da_internal"
        self.__dbUser = self.__cI.get("SITE_DB_USER_NAME")
        self.__dbPw = self.__cI.get("SITE_DB_PASSWORD")
        self.__dbSocket = self.__cI.get("SITE_DB_SOCKET")
        self.__dbPort = int(self.__cI.get("SITE_DB_PORT_NUMBER"))
        #
        self.__dbApi = DbApiUtil(dbServer=self.__dbServer, dbHost=self.__dbHost, dbName=self.__dbName, dbUser=self.__dbUser, dbPw=self.__dbPw,
                                 dbSocket=self.__dbSocket, dbPort=self.__dbPort, verbose=self.__verbose, log=self.__lfh)
        self.__dbApi.setSchemaMap(self.__schemaMap)

    # def __getDataDir(self, key, parameter):
    #     rlist = self.__dbApi.selectData(key=key, parameter=parameter)
    #     if rlist:
    #         return rlist[0]
    #     #
    #     return None

    def AuditHistory(self, depositionid=None):
        if not depositionid:
            return None
        #
        return self.__dbApi.selectData(key="AUDIT_HISTORY", parameter=(depositionid))

    def ContactAuthor(self, depositionid=None):
        if not depositionid:
            return None
        #
        if isinstance(depositionid, list):
            return self.__dbApi.selectData(key="CONTACT_AUTHOR_LIST", parameter=("', '".join(depositionid)))
        #
        return self.__dbApi.selectData(key="CONTACT_AUTHOR", parameter=(depositionid))

    def ContactAuthorPI(self, depositionid=None):
        if not depositionid:
            return None
        #
        if isinstance(depositionid, list):
            return self.__dbApi.selectData(key="CONTACT_AUTHOR_PI_LIST", parameter=("', '".join(depositionid)))
        #
        return self.__dbApi.selectData(key="CONTACT_AUTHOR_PI", parameter=(depositionid))

    def getReminderList(self, initial=None):
        if not initial:
            return None
        #
        tlist = self.__dbApi.selectData(key="GET_REMINDER_LIST", parameter=(initial))
        if not tlist:
            return None
        #
        id_list = []
        for row in tlist:
            id_list.append(row['structure_id'])
        #
        return id_list

    def getDailyStatsList(self, date=None):
        initial_list = []
        if not date:
            return initial_list
        #
        rlist = self.__dbApi.selectData(key="GET_DAILY_STATS", parameter=(date))
        return self.__getAnnoList(rlist)

    def getRangeStatsList(self, startdate=None, enddate=None):
        initial_list = []
        if not startdate or not enddate:
            return initial_list
        #
        rlist = self.__dbApi.selectData(key="GET_RANGE_STATS", parameter=(startdate, enddate))
        return self.__getAnnoList(rlist)

    def getInProcessStatsList(self):
        return self.__dbApi.selectData(key="GET_INPROCESS_STATS", parameter=())

    def getReleaseDate(self, id_string):
        emReleaseDateMap = {}
        em_rows = self.__dbApi.selectData(key='GET_EM_RELEASE_DATE', parameter=(id_string))
        if em_rows:
            for row in em_rows:
                if ('structure_id' not in row) or (not row['structure_id']) or ('date_of_EM_release' not in row) or \
                   (not row['date_of_EM_release']):
                    continue
                #
                emReleaseDateMap[row['structure_id']] = str(row['date_of_EM_release']).replace(' 00:00:00', '')
            #
        #
        releaseDateMap = {}
        rows = self.__dbApi.selectData(key='GET_RELEASE_DATE', parameter=(id_string))
        if rows:
            for row in rows:
                if ('structure_id' not in row) or (not row['structure_id']):
                    continue
                #
                if ('pdb_id' in row) and row['pdb_id']:
                    if ('date_of_RCSB_release' in row) and row['date_of_RCSB_release']:
                        releaseDateMap[row['structure_id']] = str(row['date_of_RCSB_release'])
                    #
                elif row['structure_id'] in emReleaseDateMap:
                    releaseDateMap[row['structure_id']] = emReleaseDateMap[row['structure_id']]
                #
            #
        #
        return releaseDateMap

    def GetReplaceCounts(self):
        #
        return self.__dbApi.selectData(key="GET_REPLACE_COUNTS")

    def getLigandIdList(self, entryIdList=None):
        #
        if not entryIdList:
            return None
        #
        return self.__dbApi.selectData(key="GET_LIGAND_ID_LIST", parameter=("', '".join(entryIdList)))

    def getPdbExtIdMap(self, pdbIdList):
        pdbExtIdMap = {}
        if len(pdbIdList) > 0:
            rows = self.__dbApi.selectData(key="GET_EXT_PDB_ID_INFO", parameter=("', '".join(pdbIdList)))
            for row in rows:
                if ('database_code' in row) and row['database_code'] and ('pdbx_database_accession' in row) and row['pdbx_database_accession']:
                    pdbExtIdMap[row['database_code']] = row['pdbx_database_accession']
                #
            #
        #
        return pdbExtIdMap

    def runSelectSQL(self, sql):
        return self.__dbApi.runSelectSQL(sql)

    def runUpdate(self, table=None, where=None, data=None):
        return self.__dbApi.runUpdate(table=table, where=where, data=data)

    def __getAnnoList(self, rlist):
        initial_list = []
        if rlist:
            for row in rlist:
                initial_list.append(row['rcsb_annotator'])
            #
        #
        return initial_list



##
# File:  WorkflowXMLLoader.py
# Date:  12-June-2015
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import os
import sys
from xml.dom import minidom
#
from wwpdb.utils.config.ConfigInfo import ConfigInfo
from wwpdb.apps.workmanager.workflow_access.OrderedDict import OrderedDict


class WorkflowXMLLoader(object):
    """
    """
    def __init__(self, siteId=None, verbose=False, log=sys.stderr):  # pylint:  disable=unused-argument
        """
        """
        self.__siteId = siteId
        # self.__verbose = verbose
        # self.__lfh = log
        self.__cI = ConfigInfo(self.__siteId)
        self.__workFlowXMLPath = self.__cI.get("SITE_WF_XML_PATH")
        #
        self.__metaDataInfo = {}
        self.__workFlowInfo = OrderedDict()

    def loadWorkFlowXMLFile(self, XMLFile=None):
        if not XMLFile:
            return
        #
        xmlfile = os.path.join(self.__workFlowXMLPath, XMLFile)
        if not os.access(xmlfile, os.F_OK):
            return
        #
        xmlDoc = minidom.parse(xmlfile)
        self.__readMetaDataInfo(xmlDoc)
        self.__readWorkFlowInfo(xmlDoc)

    def getMetaDataInfo(self):
        return self.__metaDataInfo

    def getWorkFlowInfo(self):
        return self.__workFlowInfo

    def __readMetaDataInfo(self, xmlDoc):
        metadata_version = xmlDoc.getElementsByTagName("wf:version")
        for item in ('author', 'major', 'date', 'id', 'name'):
            self.__metaDataInfo[item] = str(metadata_version[0].getAttribute(item))
        #

    def __readWorkFlowInfo(self, xmlDoc):
        index = 0
        #
        node = xmlDoc.getElementsByTagName("wf:entryPoint")
        task = self.__parseTaskNode(node[0])
        task['type'] = 'Entry-point'
        self.__workFlowInfo.insert(index, task['taskID'], task)
        index += 1

        nodes = xmlDoc.getElementsByTagName("wf:task")
        for node in nodes:
            task = self.__parseTaskNode(node)
            self.__workFlowInfo.insert(index, task['taskID'], task)
            index += 1
        #
        node = xmlDoc.getElementsByTagName("wf:exitPoint")
        task = self.__parseTaskNode(node[0])
        task['type'] = 'Exit-point'
        self.__workFlowInfo.insert(index, task['taskID'], task)

    def __parseTaskNode(self, node):
        task = {}
        for item in ('taskID', 'name', 'nextTask', 'breakpoint', 'reference', 'exceptionID'):
            if node.hasAttribute(item):
                task[item] = str(node.getAttribute(item))
            #
        #
        if node.hasChildNodes():
            for childnode in node.childNodes:
                if childnode.nodeType != node.ELEMENT_NODE:
                    continue
                #
                if childnode.tagName == 'wf:description':
                    task['description'] = str(childnode.firstChild.data)
                elif childnode.tagName == 'wf:workflow':
                    task['type'] = 'workflow'
                    task['file'] = str(childnode.getAttribute('file'))
                    task['classID'] = str(childnode.getAttribute('classID'))
                else:
                    for item in ('process', 'decision', 'manual'):
                        if childnode.tagName == 'wf:' + item:
                            task['type'] = item
                        #
                    #
                #
            #
        #
        return task


if __name__ == '__main__':
    loader = WorkflowXMLLoader(siteId='WWPDB_DEPLOY_TEST', verbose=True, log=sys.stderr)
    loader.loadWorkFlowXMLFile('Annotation.xml')
    info = loader.getMetaDataInfo()
    print(info)
    wfinfo = loader.getWorkFlowInfo()
    print(wfinfo[0:1])
    print(wfinfo['T1'])
    print(wfinfo)
    for key, mytask in wfinfo.items():
        print(key)
        print(mytask)
    #
    values = wfinfo.values()
    for v in values:
        print(v)



##
# File:  DepictSnapShot.py
# Date:  25-Mar-2016
#
# Updates:
#  09-Dec-2024  zf   call contentDB.getPdbExtIdMap() method to get 'ext_pdb_id'
#
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import os
import sys

from wwpdb.apps.workmanager.db_access.ContentDbApi import ContentDbApi
from wwpdb.apps.workmanager.db_access.StatusDbApi import StatusDbApi
from wwpdb.apps.workmanager.depict.DepictBase import processPublicIDs
from wwpdb.apps.workmanager.depict.ReadConFigFile import ReadConFigFile
from wwpdb.apps.workmanager.file_access.SnapShotDiff import SnapShotDiff


class DepictSnapShot(object):
    """
    """
    def __init__(self, reqObj=None, verbose=False, log=sys.stderr):
        """
        """
        self.__reqObj = reqObj
        self.__verbose = verbose
        self.__lfh = log
        self.__depositionid = str(self.__reqObj.getValue("identifier"))
        #
        self.__snapshot_tmplt = ''
        self.__snapshot_diff_tmplt = ''
        self.__snapshot_row_tmplt = ''
        self.__pageContext = ''
        self.__generatePageContent()

    def getPageText(self):
        """
        """
        return self.__pageContext

    def __generatePageContent(self):
        """
        """
        readUtil = ReadConFigFile(reqObj=self.__reqObj, configFile='allfile_config.cif', verbose=self.__verbose, log=self.__lfh)
        configDict = readUtil.read()
        if 'page_template' in configDict:
            if 'snapshot_tmplt' in configDict['page_template']:
                self.__snapshot_tmplt = configDict['page_template']['snapshot_tmplt']['page']
            #
            if 'snapshot_diff_tmplt' in configDict['page_template']:
                self.__snapshot_diff_tmplt = configDict['page_template']['snapshot_diff_tmplt']['page']
            #
            if 'snapshot_row_tmplt' in configDict['page_template']:
                self.__snapshot_row_tmplt = configDict['page_template']['snapshot_row_tmplt']['page']
            #
        #
        if not self.__snapshot_tmplt:
            return
        #
        myD = {}
        statusDB = StatusDbApi(siteId=self.__reqObj.getValue("WWPDB_SITE_ID"), verbose=self.__verbose, log=self.__lfh)
        if statusDB:
            myD = statusDB.getDepInfo(depositionid=self.__depositionid)
        #
        pdbExtIdMap = {}
        contentDB = ContentDbApi(siteId=self.__reqObj.getValue("WWPDB_SITE_ID"), verbose=self.__verbose, log=self.__lfh)
        if contentDB:
            pdbIdList = []
            if ('pdb_id' in myD) and myD['pdb_id']:
                pdbIdList.append(myD['pdb_id'])
            #
            pdbExtIdMap = contentDB.getPdbExtIdMap(pdbIdList)
        #
        myD = processPublicIDs(myD, pdbExtIdMap)
        #
        myD['identifier'] = self.__depositionid
        #
        myD['diffs'] = ''
        ssd = SnapShotDiff(siteId=str(self.__reqObj.getValue("WWPDB_SITE_ID")), verbose=self.__verbose, log=self.__lfh)
        snap = ssd.getSnap(self.__depositionid)
        if not snap:
            myD['type'] = 'Snap not found reset, ' + self.__depositionid + ', last'
            myD['date'] = ''
            myD['time'] = ''
        else:
            try:
                words = snap.split('_')
                myD['type'] = words[0]
                myD['date'] = words[1]
                myD['time'] = words[2]
                diffs = ssd.getDifference(self.__depositionid, snap)
                if diffs:
                    myD['diffs'] = str(self.__depictDifference(diffs))
                #
            except:  # noqa: E722 pylint: disable=bare-except
                myD['type'] = snap
                myD['date'] = ''
                myD['time'] = ''
            #
        #
        self.__pageContext = self.__snapshot_tmplt % myD

    def __depictDifference(self, diff_list):
        """
        """
        if not self.__snapshot_diff_tmplt:
            return ''
        #
        text = ''
        for dif in diff_list:
            myD = {}
            myD['category'] = dif['category']
            myD['rows'] = self.__depictRows(dif['data'])
            text += self.__snapshot_diff_tmplt % myD
        #
        return text

    def __depictRows(self, data_list):
        """
        """
        if not self.__snapshot_row_tmplt:
            return ''
        #
        text = ''
        for data in data_list:
            text += self.__snapshot_row_tmplt % data
        #
        return text


if __name__ == '__main__':
    from wwpdb.utils.config.ConfigInfo import ConfigInfo
    from wwpdb.utils.session.WebRequest import InputRequest
    siteId = 'WWPDB_DEPLOY_TEST_RU'
    # siteId = 'WWPDB_DEPLOY_PRODUCTION_RU'
    os.environ["WWPDB_SITE_ID"] = siteId
    cI = ConfigInfo(siteId)
    #
    myReqObj = InputRequest({}, verbose=True, log=sys.stderr)
    myReqObj.setValue("TopSessionPath", cI.get('SITE_WEB_APPS_TOP_SESSIONS_PATH'))
    myReqObj.setValue("TemplatePath", os.path.join(cI.get('SITE_WEB_APPS_TOP_PATH'), "htdocs", "wfm", "templates"))
    myReqObj.setValue("TopPath", cI.get('SITE_WEB_APPS_TOP_PATH'))
    myReqObj.setValue("WWPDB_SITE_ID", siteId)
    myReqObj.setValue("sessionid", "4f834792f0c20756c57eb8632b4e5d1c5a022f5e")
    myReqObj.setValue("identifier", sys.argv[1])
    snapshot = DepictSnapShot(reqObj=myReqObj, verbose=True, log=sys.stderr)
    print(snapshot.getPageText())



##
# File:  CifChecker.py
# Date:  26-Apr-2017
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2016 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"

import multiprocessing
import os
import sys

from wwpdb.apps.workmanager.task_access.BaseClass import BaseClass
from rcsb.utils.multiproc.MultiProcUtil import MultiProcUtil
#


class CifChecker(BaseClass):
    def __init__(self, reqObj=None, entryList=None, verbose=False, log=sys.stderr):
        """
        """
        super(CifChecker, self).__init__(reqObj=reqObj, verbose=verbose, log=log)
        self.__entryList = entryList
        self.__dictRoot = os.path.abspath(self._cICommon.get_mmcif_dict_path())
        self.__dictName = self._cICommon.get_mmcif_archive_current_dict_filename() + ".sdb"
        self.__option = self._reqObj.getValue("option")
        #
        self.__optionDict = {"cifcheck" : ["CIF DICTIONARY", "_CifChecker", "_CifChecker.cif", "_CifChecker.cif-diag.log",
                                           "_CifChecker.log", "_CifChecker_cmd.log"],
                             "mischeck" : ["MISCELLANEOUS", "_MisChecker", "_MisChecker.cif", "_MiscChecking.txt",
                                           "_MisChecker.log", "_MisChecker_cmd.log"]}
        #

    def run(self):
        """
        """
        if (self.__option != "cifcheck") and (self.__option != "mischeck"):
            return "No task was defined!"
        #
        numProc = int(multiprocessing.cpu_count() / 2)
        mpu = MultiProcUtil(verbose=True)
        mpu.set(workerObj=self, workerMethod="runMulti")
        mpu.setWorkingDir(self._sessionPath)
        _ok, _failList, _retLists, _diagList = mpu.runMulti(dataList=self.__entryList, numProc=numProc, numResults=1)
        return self.__getReturnMessage()

    def runMulti(self, dataList, procName, optionsD, workingDir):  # pylint: disable=unused-argument
        """
        """
        rList = []
        for entry_id in dataList:
            self.__runSingle(entry_id)
            rList.append(entry_id)
        #
        return rList, rList, []

    def __runSingle(self, entry_id):
        message, modelFile = self._getExistingArchiveFile(entry_id, "model", "pdbx", "latest")
        if message:
            self._dumpPickle(entry_id + self.__optionDict[self.__option][1], message)
            return
        #
        message = self.__checkModelFile(entry_id, modelFile)
        if message:
            self._dumpPickle(entry_id + self.__optionDict[self.__option][1], message)
        else:
            self._dumpPickle(entry_id + self.__optionDict[self.__option][1], "OK")
        #

    def __checkModelFile(self, entry_id, inputFile):
        localModelFile = entry_id + self.__optionDict[self.__option][2]
        outputFile = entry_id + self.__optionDict[self.__option][3]
        logFile = entry_id + self.__optionDict[self.__option][4]
        clogFile = entry_id + self.__optionDict[self.__option][5]
        for fileName in (localModelFile, outputFile, logFile, clogFile):
            self._removeFile(os.path.join(self._sessionPath, fileName))
        #
        if self.__option == "cifcheck":
            message = self._copyFileUtil(inputFile, os.path.join(self._sessionPath, localModelFile))
            if message:
                return message
            #
            options = " -dictSdb " + os.path.join(self.__dictRoot, self.__dictName) + " -f " + localModelFile
            cmd = self._getCmd("${DICTBINPATH}/CifCheck", "", "", "", clogFile, options)
        else:
            cmd = self._getCmd("${BINPATH}/MiscChecking", inputFile, outputFile, logFile, clogFile, "")
        #
        self._runCmd(cmd)
        #
        for fileName in (localModelFile, logFile, clogFile):
            self._removeFile(os.path.join(self._sessionPath, fileName))
        #
        return ''

    def __getReturnMessage(self):
        message = ''
        for entry_id in self.__entryList:
            pickleData = self._loadPickle(entry_id + self.__optionDict[self.__option][1])
            error = ''
            if pickleData and pickleData != 'OK':
                error = pickleData + "\n"
            #
            logFile = os.path.join(self._sessionPath, entry_id + self.__optionDict[self.__option][3])
            if os.access(logFile, os.F_OK):
                f = open(logFile, "r")
                error += f.read() + "\n"
                f.close()
            #
            if error:
                message += self.__optionDict[self.__option][0] + " CHECK FOR " + entry_id + ":\n" + error
            else:
                message += self.__optionDict[self.__option][0] + " CHECK FOR " + entry_id + ": OK.\n"
            #
            self._removePickle(entry_id + self.__optionDict[self.__option][1])
        #
        return message


if __name__ == '__main__':
    from wwpdb.utils.session.WebRequest import InputRequest
    from wwpdb.utils.config.ConfigInfo import ConfigInfo
    siteId = 'WWPDB_DEPLOY_TEST_RU'
    os.environ["WWPDB_SITE_ID"] = siteId
    cI = ConfigInfo(siteId)
    #
    myReqObj = InputRequest({}, verbose=True, log=sys.stderr)
    myReqObj.setValue("TopSessionPath", cI.get('SITE_WEB_APPS_TOP_SESSIONS_PATH'))
    myReqObj.setValue("WWPDB_SITE_ID", siteId)
    myReqObj.setValue("identifier", "G_1002030")
    myReqObj.setValue("sessionid", "88626e0cc0b1a1bbd10bb2df8a0d68573fcbd5fe")
    myentryList = ['D_8000210285', 'D_8000210286']
    checkUtil = CifChecker(reqObj=myReqObj, entryList=myentryList, verbose=False, log=sys.stderr)
    print(checkUtil.run())



##
# File:  SnapShotDiff.py
# Date:  23-June-2015
# Updates:
##
"""

This software was developed as part of the World Wide Protein Data Bank
Common Deposition and Annotation System Project

Copyright (c) 2015 wwPDB

This software is provided under a Creative Commons Attribution 3.0 Unported
License described at http://creativecommons.org/licenses/by/3.0/.

"""
__docformat__ = "restructuredtext en"
__author__ = "Zukang Feng"
__email__ = "zfeng@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.07"


import os
import sys
import pickle

from wwpdb.apps.wf_engine.engine.WFEapplications import getPicklePath


class SnapShotDiff(object):
    """
    """
    def __init__(self, siteId=None, verbose=False, log=sys.stderr):  # pylint: disable=unused-argument
        """
        """
        # self.__siteId = siteId
        # self.__verbose = verbose
        self.__lfh = log

    def getSnap(self, depID):
        """ Method to return the file handle path for a file system snapshot
            name is the type of snapshot like reset or upload
            Only the last snapshot is supported at this time
        """
        path = getPicklePath(depID)
        if not os.access(path, os.F_OK):
            return None
        #
        snap = [temp for temp in os.listdir(path) if os.path.isdir(os.path.join(path, temp)) and temp.startswith('reset')]
        #
        if not snap:
            return None
        #
        snap.sort(key=lambda x: os.stat(os.path.join(path, x)).st_mtime)
        return snap[-1]

    def getDifference(self, depID, snap):
        """ Method to check all categories in a snapshot and return all differences
            Look for all pickle files - and compare each matching file name
        """
        path = getPicklePath(depID)
        #
        ret = []
        dlist = os.listdir(path)
        for filename in dlist:
            if filename[-4:] == '.pkl':
                olddat = self.__getData(os.path.join(path, filename))
                newdat = self.__getData(os.path.join(path, snap, filename))
                if olddat and newdat:
                    diff = self.__difData(filename, olddat, newdat)
                    if diff and ('data' in diff) and diff['data']:
                        ret.append(diff)
                    #
                #
            #
        #
        return ret

    def __getData(self, filename):
        """ Get an object as the contents of a named pickle file
        """
        try:
            f = open(filename, 'rb')
            ret = pickle.load(f)
            f.close()
            return ret
        except Exception as e:
            self.__lfh.write("Exception=%s\n" % str(e))
            return None
        #

    def __difData(self, filename, olddat, newdat):
        """ Method to compare the contents of 2 category pickle files
            Returns a dictionary of differences
        """
        ret = {}
        try:
            dif = []
            if ('items' in olddat) and ('items' in newdat):
                items1 = olddat['items']
                items2 = newdat['items']
                ints1 = len(items1)
                ints2 = len(items2)
                if ints1 == ints2:
                    n = 0
                    for (i1, i2) in zip(items1, items2):
                        n = n + 1
                        for item1, value1 in i1.items():
                            if not (item1.endswith('_ordinal') or item1 == 'id') :
                                if item1 in i2.keys():
                                    value2 = i2[item1]
                                    if value1['value'] != value2['value']:
                                        d = {}
                                        d['instance'] = n
                                        d['item'] = item1
                                        d['d1'] = value1['value']
                                        d['d2'] = value2['value']
                                        dif.append(d)
                                    #
                                else:
                                    d = {}
                                    d['instance'] = n
                                    d['item'] = item1
                                    d['d1'] = "Item missing"
                                    d['d2'] = "Item missing"
                                    dif.append(d)
                                #
                            #
                        #
                    #
                else:
                    d = {}
                    d['instance'] = 0
                    d['item'] = 'Length of category has changed'
                    d['d1'] = str(ints1)
                    d['d2'] = str(ints2)
                    dif.append(d)
                #
            #
            if dif:
                ret['category'] = filename[0:-4]
                ret['data'] = dif
            #
        except Exception as e:
            self.__lfh.write("Exception=%s\n" % str(e))
        #
        return ret


def main_snap():
    ssd = SnapShotDiff(siteId="WWPDB_DEPLOY_INTERNAL")
    snap = ssd.getSnap('D_1100209960')
    print(snap)
    diff = ssd.getDifference('D_1100209960', snap)
    print(diff)


if __name__ == '__main__':
    main_snap()
